#include <grwin.h>
#include <stdio.h>
#include "mandelcommon.h"
#include "grad.h"

static int grad[GRADATION_PICS_NUM][GRAD_TONE_NUM];
static int *gradLut[GRADATION_PICS_NUM];
static int nowSelect = -1;

static int linearInterpolateLut(int *lutAddress,int tone,int seq);
static int linearInterpolateSub(int prevColor,int nextColor,int numPolate,int *addrPolate);
#ifdef DEBUG
static void dumpLut(int tone);
#endif

void gradInitforLut(void){
  int i;
  for(i=0;i<GRADATION_PICS_NUM;i++){
    gradLut[i] = NULL;
  }
  
  return;
}

void gradEnd(void){
  int i;
  for(i=0;i<GRADATION_PICS_NUM;i++){
    if(gradLut[i] != NULL){
      free(gradLut[i]);
    }
  }
  return;
}

int gradMakeLut(int tone){
  int i;
  
  for(i=0;i<GRADATION_PICS_NUM;i++){
    gradLut[i] = malloc(sizeof(int)*tone);
    if(gradLut[i] == NULL){
      fprintf(stderr,"failed allocating for LUT\n");
      return -1;
    }
  }
  for(i=0;i<GRADATION_PICS_NUM;i++){
    linearInterpolateLut(gradLut[i],tone,i);
  }
#ifdef DEBUG
  dumpLut(tone);
#endif
  return 0;
}

static int linearInterpolateLut(int *lutAddress,int tone,int seq){
  int i,j;
  double space;
  int prevColor,nextColor;
  int *isFilled;

  isFilled = calloc(tone,sizeof(int));
  if(isFilled == NULL){
    return -1;
  }
  
  space = (double)tone / GRAD_TONE_NUM;
  
  /* 最初と最後は変えない */
  *(lutAddress + tone - 1) = grad[seq][GRAD_TONE_NUM-1];
  isFilled[tone-1] = 1;
  for(i=0;i<GRAD_TONE_NUM;i++){
    *(lutAddress+(int)(i*space)) = grad[seq][i];
    isFilled[(int)(i*space)] = 1;
  }
  
  for(i=0;i<tone;i++){
    if(isFilled[i] == 0){
      prevColor = *(lutAddress + i - 1);
      for(j=1;;j++){
	if(isFilled[i + j] == 1){
	  nextColor = *(lutAddress + i + j);
	  break;
	}
	isFilled[i + j] = 1;
      }
      linearInterpolateSub(prevColor,nextColor,j,lutAddress + i);
    }
  }
    
  free(isFilled);
  return 0;
}

static int linearInterpolateSub(int prevColor,int nextColor,int numPolate,int *addrPolate){
  int prevR,prevG,prevB;
  int nextR,nextG,nextB;
  int dR,dG,dB;
  int i;
  prevR = (prevColor >> 0x00) & 0xff;
  prevG = (prevColor >> 0x08) & 0xff;
  prevB = (prevColor >> 0x10) & 0xff;

  nextR = (nextColor >> 0x00) & 0xff;
  nextG = (nextColor >> 0x08) & 0xff;
  nextB = (nextColor >> 0x10) & 0xff;
  
  dR = (nextR-prevR)/(numPolate + 1);
  dG = (nextG-prevG)/(numPolate + 1);
  dB = (nextB-prevB)/(numPolate + 1);
  
  for(i=0;i<numPolate;i++){
    *(addrPolate + i) = (prevB+dB*(i+1) << 0x10) | (prevG+dG*(i+1) << 0x08) | (prevR+dR*(i+1) << 0x00);
  }

  return 0;
}

/*************************************
 グラデーション表示枠のビューポート、ワールド座標系設定
 return:
 無し
 ************************************/
void gradBarInit(int num){
  int i;
  for(i = 0;i < num;i++){
    GWselvp(INITIAL_VPORT);
    GWport(DRAW_RECT_GRAD_BAR_SHIFT + DRAW_RECT_MARGIN32 * 3 + DRAW_RECT_PIXEL + DRAW_RECT_INNER_MARGIN8 * 2 + DRAW_RECT_SUB_PIXEL + DRAW_RECT_INNER_MARGIN2 * 2,WINDOW_HEIGHT - DRAW_RECT_MARGIN16 - DRAW_RECT_BAR_HEIGHT * (i + 1) - DRAW_RECT_BAR_MARGIN * i,
	   DRAW_RECT_MARGIN32 * 3 + DRAW_RECT_PIXEL + DRAW_RECT_INNER_MARGIN8 * 2 + DRAW_RECT_SUB_PIXEL + DRAW_RECT_INNER_MARGIN2 * 2 + DRAW_RECT_BAR_WIDTH - 1.f,WINDOW_HEIGHT - DRAW_RECT_MARGIN16 - (DRAW_RECT_BAR_HEIGHT * i - 1.f) - DRAW_RECT_BAR_MARGIN * i);
    GWindow(0,0,(DRAW_RECT_BAR_WIDTH - 1.f),DRAW_RECT_BAR_HEIGHT - 1.f);
    GWsavevp(COLOR_BAR_VPORT + i);
  }

  return;
}

/*************************************
 グラデーションを表示
 return:
 無し
 ************************************/
void gradShowGradation(int num){
  int ret;
  int i,j;
  int penColor;

  for(i = 0;i < num;i++){
    GWselvp(COLOR_BAR_VPORT + i);
    GWsetogn(0,-GRADATION_OGN);
    for(j = 0;j < GRAD_TONE_NUM;j++){
      penColor = GWkrgb((grad[i][j] >> 16) & 255,(grad[i][j] >> 8) & 255,grad[i][j] & 255);
      GWsetpen(penColor,1,0,GWX_COPYPEN);
      GWmove2((float)j,0);
      GWline2((float)j,DRAW_RECT_BAR_HEIGHT - 1.f);
    }
    GWflush(GRADATION_OGN);
    gradBarRectErase(i);
  }
  
  return;
}

/*************************************
 カラー表示部の枠を描く
 ************************************/
void gradBarRect(int num){
  int i;

  for(i = 0;i < num;i++){
    GWselvp(COLOR_BAR_VPORT + i);
    GWsetogn(0,-(GRADBAR_RECT_OGN + i));
    GWrect(0,0,DRAW_RECT_BAR_WIDTH - 1,DRAW_RECT_BAR_HEIGHT - 1);
    GWflush(GRADBAR_RECT_OGN + i);
  }

  return;
}

/*************************************
 グラデーション番号を表示
 return:
 無し
 ************************************/
void gradShowGradationNumber(int num){
  int i;
  char dispNum[2];
#define GRAD_MARGIN_TXT 24
#define NUM_TEXT_FONT "Consolas"
#define NUM_TEXT_OFFSET_Y 2.f

  for(i = 0;i < num;i++){
    GWselvp(COLOR_BAR_VPORT + i);
    GWsetogn(0,-(GRAD_NUM_OGN + i));
    GWsettxt(DRAW_RECT_BAR_HEIGHT,0,GWO_BOTTOMLEFT,-1,-1,NUM_TEXT_FONT,strlen(NUM_TEXT_FONT));
    sprintf(dispNum,"%d",i + 1);
    GWputtxt(DRAW_RECT_BAR_WIDTH + GRAD_MARGIN_TXT,NUM_TEXT_OFFSET_Y,dispNum,strlen(dispNum));
    GWflush(GRAD_NUM_OGN + i);
  }

  return;
}

/*************************************
 グラデーション番号を選択
 return:
 -1:変更なし
 ************************************/
int gradSelectGradationNumber(int num){
  char dispNum[2];

  if(num == nowSelect){
    return -1;
  }
  if(nowSelect == -1){
    nowSelect++;
  }

  GWerase(GRAD_NUM_OGN + num,1);
  GWerase(GRAD_NUM_OGN + nowSelect,1);

  GWselvp(COLOR_BAR_VPORT + nowSelect);
  GWsetogn(0,-(GRAD_NUM_OGN + nowSelect));
  GWsettxt(DRAW_RECT_BAR_HEIGHT,0,GWO_BOTTOMLEFT,GWC_FGCOLOR,GWC_BKCOLOR,NUM_TEXT_FONT,strlen(NUM_TEXT_FONT));
  sprintf(dispNum,"%d",nowSelect + 1);
  GWputtxt(DRAW_RECT_BAR_WIDTH + GRAD_MARGIN_TXT,NUM_TEXT_OFFSET_Y,dispNum,strlen(dispNum));

  GWselvp(COLOR_BAR_VPORT + num);
  GWsetogn(0,-(GRAD_NUM_OGN + num));
  GWsettxt(DRAW_RECT_BAR_HEIGHT,0,GWO_BOTTOMLEFT,GWC_BLACK,GWC_DARKGRAY,NUM_TEXT_FONT,strlen(NUM_TEXT_FONT));
  sprintf(dispNum,"%d",num + 1);
  GWputtxt(DRAW_RECT_BAR_WIDTH + GRAD_MARGIN_TXT,NUM_TEXT_OFFSET_Y,dispNum,strlen(dispNum));
    
  GWflush(GRAD_NUM_OGN + num);
  GWflush(GRAD_NUM_OGN + nowSelect);

  nowSelect = num;
  return 0;
}

/*************************************
 tgaを読み取んで階調を取得
 return:
 成功: 0
 失敗: 負
 ************************************/
int gradInitGradArray(int num){
  int i;
  char fileName[64];
  int ret;

  for(i = 0;i < num;i++){
    sprintf(fileName,"%s%d.tga",GRADATION_PICS_NAME,i + 1);
    if(ret = loadTga(fileName,grad[i],GRAD_TONE_NUM) != 0){
      return ret;
    }
  }
  return 0;
}

/*************************************
 カラー表示部の枠を消す
 ************************************/
void gradBarRectErase(int num){
  GWselvp(COLOR_BAR_VPORT + num);
  GWerase(GRADBAR_RECT_OGN + num,1);
    
  return;
}

int *gradGetGrad(void){
  return gradLut[nowSelect];
  //return grad[nowSelect];
}

int gradGetSelecting(void){
  return nowSelect;
}

#ifdef DEBUG
static void dumpLut(int tone){
  int i;
  int color;
  
  for(i=0;i<tone;i++){
    color=gradLut[0][i];
    printf("%4d: R=%d, G=%d, B=%d\n",i,(color >> 0x00) & 0xff,(color >> 0x08) & 0xff,(color >> 0x10) & 0xff);
  }

  return;
}
#endif
