#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

#define TGA_DIR_PATH "gradTga/"

static int tgaReadImage(FILE *fp,unsigned char *imgBuff,int buffSize);
static int tgaReadImageRle(FILE *fp,unsigned char *imgBuff,int buffSize,unsigned char byPP);
static int tgaWriteTone(int *dest,unsigned char *imgBuff,unsigned short tgaWidth,unsigned short tgaHeight,unsigned char byPP,int isUpToDown);

/***************************************
 szFileNametga摜KdestɎ荞
 return:
 : 0
 s: ̒l
 **************************************/
int loadTga(char *szFileName,int *dest,int scale){
  FILE *fp;
  char pathFile[64];
  unsigned char header[18];
  int isRle = 0;
  unsigned short tgaWidth,tgaHeight;
  unsigned char biPP,byPP;
  int isUpToDown;
  int ret;
  unsigned char *imgBuff;
  int imgBuffSize;
  
  if(strlen(TGA_DIR_PATH) + strlen(szFileName) + 1 > sizeof(pathFile) / sizeof(char)){
    return -1;
  }
  sprintf(pathFile,"%s%s",TGA_DIR_PATH,szFileName);
  if((fp = fopen(pathFile,"rb")) == NULL){
    return -2;
  }

  ret = -3;
  if(fread(header,sizeof(header),1,fp) != 1){
    goto loadTga_err;
  }

  /* 0x0a, 0x0bRLEkĂ */
  if(header[2] == 0x0a || header[2] == 0x0b){
    isRle = 1;
  }
  memcpy(&tgaWidth,header + 12,2);
  memcpy(&tgaHeight,header + 14,2);

  if(tgaHeight != scale){
    fprintf(stderr,"image height must be %d\n",scale);
    goto loadTga_err;
  }

  biPP = header[16];
  isUpToDown = (header[17] >> 5) & 1;

#ifdef DEBUG
  fprintf(stderr,"******************************\n");
  fprintf(stderr,"loadTga: pathFile = %s\n",pathFile);
  fprintf(stderr,"loadTga: isRle = %d\n",isRle);
  fprintf(stderr,"loadTga: tgaWidth = %d\n",tgaWidth);
  fprintf(stderr,"loadTga: tgaHeight = %d\n",tgaHeight);
  fprintf(stderr,"loadTga: biPP = %d\n",biPP);
  fprintf(stderr,"loadTga: isUpToDown = %d\n",isUpToDown);
#endif
  
  byPP = biPP / CHAR_BIT;
  imgBuffSize = tgaWidth * tgaHeight * byPP;
  if((imgBuff = malloc(imgBuffSize)) == NULL){
    goto loadTga_err;
  }
  if(!isRle){
    ret = tgaReadImage(fp,imgBuff,imgBuffSize);
  }else{
    ret = tgaReadImageRle(fp,imgBuff,imgBuffSize,byPP);
  }
  if(ret != 0){
    goto loadTga_err;
  }
  fclose(fp);

  tgaWriteTone(dest,imgBuff,tgaWidth,tgaHeight,byPP,isUpToDown);

  free(imgBuff);  

  return 0;

 loadTga_err:
  if(imgBuff != NULL){
    free(imgBuff);
  }
  fclose(fp); 
  return ret;
 }

/*******************************************
 t@C|C^C[Wǂݎ
 (ʏ)
 return:
 : 0
 s: -1
 ******************************************/
static int tgaReadImage(FILE *fp,unsigned char *imgBuff,int buffSize){
  if(fread(imgBuff,buffSize,1,fp) != 1){
    return -1;
  }

  return 0;
}

/*******************************************
 t@C|C^C[Wǂݎ
 (RLEk)
 return:
 : 0
 s: -1
 ******************************************/
static int tgaReadImageRle(FILE *fp,unsigned char *imgBuff,int buffSize,unsigned char byPP){
  int isRepeat;
  unsigned char iterate;
  int i = 0,j,maxI;
  unsigned char byte;
  int color;

  if(byPP > sizeof(int)){
    return -1;
  }
  if(imgBuff == NULL){
    return -1;
  }
  maxI = buffSize / byPP;

  while(i < maxI){
    if(fread(&byte,1,1,fp) != 1){
      return -1;
    }

    isRepeat = byte >> 7;
    iterate = (byte & 127) + 1;

    if(isRepeat){
      if(fread(&color,byPP,1,fp) != 1){
	return -1;
      }
      for(j = 0;j < iterate && i < maxI;j++,i++){
	memcpy(imgBuff,&color,byPP);
	imgBuff += byPP;
      }
    }else{
      for(j = 0;j < iterate && i < maxI;j++,i++){
	if(fread(&color,byPP,1,fp) != 1){
	  return -1;
	}
	memcpy(imgBuff,&color,byPP);
	imgBuff += byPP;
      }
    }
  }

  return 0;
}

/***********************************************
 Kݒ
 return:
 0
 **********************************************/
static int tgaWriteTone(int *dest,unsigned char *imgBuff,unsigned short tgaWidth,unsigned short tgaHeight,unsigned char byPP,int isUpToDown){
  int diff;
  int i;

  if(!isUpToDown){
    imgBuff += byPP * (tgaWidth * tgaHeight - 1);
    diff = -(byPP * tgaWidth);
  }else{
    diff = byPP * tgaWidth;
  }
  
  for(i = 0;i < tgaHeight;i++){
    memcpy(dest + i,imgBuff,byPP);
    imgBuff += diff;
  }

  return 0;
}
