#include <stdio.h>
#include "mList.h"

/******************************
リストデスクリプタの初期化
 return:
 無し
******************************/
void mListInit(pMDesc list){
  list->next = NULL;
  list->prev = NULL;
  list->data = (void *)0;
  return;
}

/******************************
 ノード数を取得
 return:
 ノード数
******************************/
int mListCount(pMDesc list){
  return (int)list->data;
}

/******************************
リストの最後にノードを追加
 return:
 無し
******************************/
void mListAdd(pMDesc list,pMNode node){
  node->next = NULL;
  node->prev = mListLast(list);
  if(node->prev != NULL){
    node->prev->next = node;
  }else{
    list->next = node;
  }
  list->prev = node;

  (int)list->data++;
  
  return;
}

/******************************
リストの先頭にノードを追加
 return:
 無し
******************************/
void mListAddFirst(pMDesc list,pMNode node){
  node->next = mListFirst(list);
  node->prev = NULL;
  if(node->next != NULL){
    node->next->prev = node;
  }else{
    list->prev = node;
  }
  list->next = node;
  
  (int)list->data++;
  
  return;
}

/******************************
 リストの先頭からノードを取得
 取得後リストから削除される。
 return:
 ノードアドレス(ノードが無い場合はNULL)
******************************/
pMNode mListGet(pMDesc list){
  pMNode node;

  node = mListFirst(list);
  if(node != NULL){
    list->next = node->next;
    if(node->next != NULL){
      node->next->prev = NULL;
    }else{
      list->prev = NULL;
    }
    (int)list->data--;
  }

  return node;
}

/***********************************************
 リストから指定されたノードを削除する
 return:
 OK: 削除したnodeアドレス
 指定のノードは見つからない: NULL
 **********************************************/
pMNode mListDelete(pMDesc list,pMNode delNode){
  pMNode node;

  node = mListFirst(list);
  while((node != delNode) && (node != NULL)){
    node = mListNext(node);
  }
  if(node == NULL){
    goto mListDelete_notFoundNode;
  }
  if(node->next != NULL){
    if(node->prev != NULL){
      node->next->prev = node->prev;
      node->prev->next = node->next;
    }else{
      list->next = node->next;
    }
  }else{
    if(node->prev != NULL){
      list->next = NULL;
      list->prev = NULL;
    }else{
      node->prev->next = NULL;
      list->prev = node->prev;
    }
  }
  (int)list->data--;
   
 mListDelete_notFoundNode:
  return node;
}

/******************************
 return:
 次のノードのアドレス
******************************/
pMNode mListNext(pMNode node){
  return node->next;
}

/******************************
 return:
 前のノードのアドレス
******************************/
pMNode mListPrev(pMNode node){
  return node->prev;
}

/******************************
 return:
 リストの先頭のノードアドレス
******************************/
pMNode mListFirst(pMDesc list){
  return list->next;
}

/******************************
 return:
 リストの後端のノードアドレス
******************************/
pMNode mListLast(pMDesc list){
  return (pMNode)list->prev;
}

#ifdef DEBUG
/******************************
リストの状態を表示(デバッグ用)
 return:
 無し
******************************/
void mListShow(pMDesc list){
  pMNode node;

  fprintf(stderr,"show Node*************************\n");
  node = mListFirst(list);
  while(node != NULL){
    fprintf(stderr,"node: %p, prev: %p, next: %p\n",node,node->prev,node->next);
    node = mListNext(node);
  }
  fprintf(stderr,"count = %d\n",(int)list->data);

  return;
}
#endif
