#include <grwin.h>
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>
#include <process.h>
#include "initGr.h"
#include "mList.h"
#include "thread.h"
#include "status.h"
#include "mandel.h"
#include "brot.h"
#include "grad.h"
#include "mandelcommon.h"

#define STACKSIZE_PROGRESS_THREAD 0x400
#define DEFAULT_MAX_ITERATION 255
#define THREADS_MAX 64
#define INITIAL_CENTER_X -0.5
#define INITIAL_CENTER_Y 0.0
#define INITIAL_WIDTH 3.0

typedef struct _argParam{
  int threadNum;
  int isWhiteBackColor;
  int iterateNum;
}argParam,*pArgParam;

static int set[5];
static HANDLE grDrawMutex;
static HANDLE nodeMutex;
static HANDLE eventFlag;

#ifdef DEBUG
static void DEBUG_showParam(pArgParam param);
static void DEBUG_dumpBrot(int *brot);
static void DEBUG_dump__(mandel *mandel);
#endif

static void initParam(pArgParam param);
static void usage(void);
static int getParam(int argc,char **argv,pArgParam param);
static int getArgNum(char *strNum);
static int isValidOption(char opt,const char *validParam,int validParamNum);
static int throwTask(int iterateNum,int ThreadNum,int isWhiteBackColor,mandel *mandel);
static int allocBrot(int **brot,int *set);
static void freeBrot(int **brot);
static void zeroFillBrot(int **brot,int i);
static int initializeMandel(mandel *mandel,int threadNum);
static void freeMandel(mandel *mandel);
static void watchProgressThread(mandel *mandel);
static void drawLineThread(mandel *mandel);
static void delBitmapLine(mandel *mandel);
static void flipSet(int stackId);
static void setCenter(mandel *mandel,float x,float y);
static void copyCt(mandel *mandel,int stackId);

int main(int argc,char **argv){
  argParam param;
  mandel mandel;
  volatile int i;
  float x,y,prevX,prevY;
  unsigned int magRect = 0x40;
  int flgRedrawRect = 0;
  int bmpStackId = 1; /* Oobt@ */
  unsigned int tmp;
  char tmpStr[64] = "";
  int input;

  /*
  printf("a= %d\n",sizeof(long double));
  printf("a= %d\n",sizeof(double));
  printf("a= %d\n",sizeof(__float128));
  */

  initParam(&param);
  gradInitforLut();

  
  if(getParam(argc,argv,&param)){
    usage();
    return -1;
  }

  if(initializeMandel(&mandel,param.threadNum) != 0){
    fprintf(stderr,"error: initializeMandel\n");
    return -1;
  }
  
#ifdef DEBUG
  DEBUG_showParam(&param);
#endif

  initWindow(param.isWhiteBackColor);
  statusShow(param.threadNum,param.iterateNum,1);
  gradMakeLut(DEFAULT_MAX_ITERATION+1);
  statusSetCheck(1);

 REDO:

  if(threadInit(param.threadNum,&mandel.taskQ) != 0){
    fprintf(stderr,"error: threadInit\n");
    freeMandel(&mandel);
    return -1;
  }

  if(throwTask(param.iterateNum,param.threadNum,param.isWhiteBackColor,&mandel) != 0){
    /* taskque炷ׂĎoāAeƃm[hf[g */
    fprintf(stderr,"error: throwTask\n");
    freeMandel(&mandel);
    return -1;
  }

  //getchar();

  statusCenterCoord(&mandel,set[0]);

  /*
  a = brotMakeBitmap(mandel.brot[set[0]]);
  GWselvp(MAIN_VPORT);
  GWsetbmp(a,DRAW_RECT_PIXEL,DRAW_RECT_PIXEL,param.isWhiteBackColor? GWY_INVERTNOT:GWY_SRCINVERT,0,GWO_BOTTOMLEFT);
  GWsetogn(0,-MAIN_BMP_OGN);
  GWputbmp(a,0,0,1);
  GWflush(MAIN_BMP_OGN);
  */
  //static void watchProgressThread(mandel *mandel);

#ifdef DEBUG
  DEBUG_dump__(&mandel);
#endif


  _beginthreadex(NULL,STACKSIZE_PROGRESS_THREAD,(void *)watchProgressThread,(void *)&mandel,0,NULL);
  _beginthreadex(NULL,STACKSIZE_PROGRESS_THREAD,(void *)drawLineThread,(void *)&mandel,0,NULL);

  startThread(param.threadNum);

  /*
  while(1){
    if(WAIT_OBJECT_0 == WaitForSingleObject(eventFlag,10)){
      break;
    }
    WaitForSingleObject(grDrawMutex,INFINITE);
    GWsetogn(0,-MAIN_BMP_OGN);
    ReleaseMutex(grDrawMutex);    
  }
  */
  
  WaitForSingleObject(eventFlag,INFINITE); /* drawLineThreadI҂ */

  brotMakeBitmapBuffer(mandel.pixBuff,mandel.brot[set[0]]);
  mandel.bmp2[set[0]] = GWmakebmp(0,PIXEL_X,PIXEL_X,32,mandel.pixBuff);
  GWselvp(MAIN_VPORT);
  GWsetogn(0,-MAIN_BMP_OGN);

  GWsetbmp(mandel.bmp2[set[0]],DRAW_RECT_PIXEL,DRAW_RECT_PIXEL,param.isWhiteBackColor? GWY_INVERTNOT:GWY_SRCINVERT,0,GWO_BOTTOMLEFT);

  GWerase(MAIN_BMP_OGN,0);
  delBitmapLine(&mandel);

  GWputbmp(mandel.bmp2[set[0]],0,0,1);
  GWflush(MAIN_BMP_OGN);

  while(1){

    switch(keyGetStatus(&x,&y)){
    case -1:

      break;
    case 0:
      GWdelbmp(mandel.bmp2[set[0]]);
      brotMakeBitmapBuffer(mandel.pixBuff,mandel.brot[set[0]]);
      mandel.bmp2[set[0]] = GWmakebmp(0,PIXEL_X,PIXEL_X,32,mandel.pixBuff);
      GWselvp(MAIN_VPORT);
      GWerase(MAIN_BMP_OGN,1);
      GWsetogn(0,-MAIN_BMP_OGN);
      GWsetbmp(mandel.bmp2[set[0]],DRAW_RECT_PIXEL,DRAW_RECT_PIXEL,param.isWhiteBackColor? GWY_INVERTNOT:GWY_SRCINVERT,0,GWO_BOTTOMLEFT);
      GWputbmp(mandel.bmp2[set[0]],0,0,1);
      GWflush(MAIN_BMP_OGN);
      mandel.oct[set[0]].color = gradGetSelecting();
      break;
    case 1:
      //printf("%f %f\n",x,y);
      GWerase(MAIN_BMP_OGN,1);
      GWerase(MAIN_BMP_OGN + bmpStackId,1);
      if(mandel.bmp2[set[bmpStackId]] != 0){
	GWdelbmp(mandel.bmp2[set[bmpStackId]]);
      }
      //GWdelbmp(mandel.bmp2[set[bmpStackId]]);
      GWselvp(SUB_VPORT_BOTTOM + bmpStackId - 1);
      GWsetogn(0,-MAIN_BMP_OGN - bmpStackId);
      GWsetbmp(mandel.bmp2[set[0]],DRAW_RECT_PIXEL,DRAW_RECT_PIXEL,param.isWhiteBackColor? GWY_INVERTNOT:GWY_SRCINVERT,0,GWO_BOTTOMLEFT);
      GWputbmp(mandel.bmp2[set[0]],0,0,1);
      GWflush(MAIN_BMP_OGN + bmpStackId);

      GWerase(MAG_RECT_OGN,1);
      
      GWselvp(MAIN_VPORT);
      
      progressClearProgress();
      progressShowStatus(PROGRESS_CALCULATING);
      GWerase(STATUS_CENTER_COORD,1);

      if(x + magRect > DRAW_RECT_PIXEL){
	x = DRAW_RECT_PIXEL - magRect - 1.f;
      }else if(x - magRect < 0){
	x = 0 + magRect;
      }
      if(y + magRect > DRAW_RECT_PIXEL){
	y = DRAW_RECT_PIXEL - magRect - 1.f;
      }else if(y - magRect < 0){
	y = 0 + magRect;
      } 


      tmp = 0x10;
      i=0;      
      for(i = 0;!((0x10 << i) & magRect);i++);
      tmp >>= i;
      
      mandel.oct[set[0]].color = gradGetSelecting();
      copyCt(&mandel,set[bmpStackId]);

      flipSet(bmpStackId);

      setCenter(&mandel,x,y);

      mandel.oct[set[0]].mag *= tmp;

      statusShow(-1,-1,mandel.oct[set[0]].mag);

      //mandel.bmp2[set[bmpStackId]] = mandel.bmp2[set[0]];
      //      mandel.brot[set[bmpStackId]] = mandel.brot[set[0]];

      if(bmpStackId == 4){
	bmpStackId = 1;
      }else{
	bmpStackId++;
      }
#ifdef DEBUG
      printf("-------------------------------------------\n");
      DEBUG_dump__(&mandel);
#endif

      goto REDO;
      break;
    case 2:
      if(magRect & 0x10){
	magRect <<= 2;
	prevX = 0;
      }
      else{
	magRect >>= 1;
	prevX = 0;
      }
      break;
    case 3:
      switch(*(int *)&y){
      case 1:
      case 2:
      case 3:
      case 4:
	break;
      default:
	goto ERROR_INVALID_CLICK;
      }
#ifdef DEBUG
      printf("%d\n",*(int *)&y);
#endif
      if(mandel.bmp2[set[*(int *)&y]] == 0){
	break;
      }

      GWselvp(SUB_VPORT_BOTTOM + *(int *)&y - 1);
      GWsetogn(0,-MAIN_BMP_OGN - *(int *)&y);
      GWsetbmp(mandel.bmp2[set[0]],DRAW_RECT_PIXEL,DRAW_RECT_PIXEL,param.isWhiteBackColor? GWY_INVERTNOT:GWY_SRCINVERT,0,GWO_BOTTOMLEFT);
      GWerase(MAIN_BMP_OGN + *(int *)&y,0);
      GWputbmp(mandel.bmp2[set[0]],0,0,1);
      GWflush(MAIN_BMP_OGN + *(int *)&y);

      GWselvp(MAIN_VPORT);
      GWsetogn(0,-MAIN_BMP_OGN);
      GWsetbmp(mandel.bmp2[set[*(int *)&y]],DRAW_RECT_PIXEL,DRAW_RECT_PIXEL,param.isWhiteBackColor? GWY_INVERTNOT:GWY_SRCINVERT,0,GWO_BOTTOMLEFT);
      GWerase(MAIN_BMP_OGN,0);
      GWputbmp(mandel.bmp2[set[*(int *)&y]],0,0,1);
      GWflush(MAIN_BMP_OGN);

      flipSet(*(int *)&y);
      //DEBUG_dump__(&mandel);
      GWerase(STATUS_CENTER_COORD,1);
      statusCenterCoord(&mandel,set[0]);
      gradSelectGradationNumber(mandel.oct[set[0]].color);
      
      statusShow(-1,-1,mandel.oct[set[0]].mag);
    ERROR_INVALID_CLICK:
      break;
	    case 4:
      switch(*((int *)&y)){
      case 1:
	#define MESSAGE_INPUT_THREAD "Input thread num"
	sprintf(tmpStr,"%d",param.threadNum);
	GWinput(MESSAGE_INPUT_THREAD,strlen(MESSAGE_INPUT_THREAD),tmpStr,sizeof(tmpStr));
	input = atoi(tmpStr);
	if(input < 1 || input > THREADS_MAX){
	  fprintf(stderr,"thread num must be between [1, %d]\n",THREADS_MAX);
	}
	else{
	  param.threadNum = input;
	  mandel.threadNum = input;
	  free(mandel.pReport);
	  if((mandel.pReport = malloc(sizeof(report) * param.threadNum)) == NULL){
	    fprintf(stderr,"failed allocating memory\n");
	    freeMandel(&mandel);
	    return -1;
	  }
	  statusShow(param.threadNum,-1,-1);
	}
	break;
      case 2:
	#define MESSAGE_INPUT_ITERATE "Input max-iterate"
	sprintf(tmpStr,"%d",param.iterateNum);
	GWinput(MESSAGE_INPUT_ITERATE,strlen(MESSAGE_INPUT_ITERATE),tmpStr,sizeof(tmpStr));
	input = atoi(tmpStr);
	//if(input < 1){
	if(input < 255){
	  fprintf(stderr,"invalid iterate\n");
	  break;
	}
	if(input == param.iterateNum){
	  break; /* do nothing */
	}
	param.iterateNum = input;
	statusShow(-1,param.iterateNum,-1);
	
	gradEnd();
	gradMakeLut(input+1);
	goto REDO;
	break;
      case 3:
	#define MESSAGE_INPUT_MAG "Input mag"	
	sprintf(tmpStr,"%d",mandel.oct[set[0]].mag);
	GWinput(MESSAGE_INPUT_MAG,strlen(MESSAGE_INPUT_MAG),tmpStr,sizeof(tmpStr));
	input = atoi(tmpStr);
	if(input < 1){
	  fprintf(stderr,"invalid mag\n");
	  break;
	}
	if(input == mandel.oct[set[0]].mag){
	  break; /* do nothing */
	}
	mandel.oct[set[0]].mag = input;
	statusShow(-1,-1,input);
	goto REDO;
	break;
      case 5:
	statusSetCheck(1);
	eraseJuliaEq();
	break;
      case 6:
	statusSetCheck(0);
	showJuliaEq();
	break;
      default:
	break;
      }
      continue;
    case 5:
      switch(*((int *)&y)){
      case 0:
	goto END;
	break;
      case 1:
	sprintf(tmpStr,"%R%1.17lf_I%1.17lf_M%llu.bmp",(double)mandel.oct[set[0]].centerX,(double)mandel.oct[set[0]].centerY,mandel.oct[set[0]].mag);
	if(GWsavebmp(mandel.bmp2[set[0]],tmpStr,strlen(tmpStr)) == 1){
	  printf("saved BMP: %s\n",tmpStr);
	}
	else{
	  printf("failed saving BMP\n");
	}
	
	break;
      case 2:
	//GWdelbmp(mandel.bmp2[set[0]]);
	mandel.oct[set[0]].centerX = INITIAL_CENTER_X;
	mandel.oct[set[0]].centerY = INITIAL_CENTER_Y;
	mandel.initialWidth = INITIAL_WIDTH;
	mandel.oct[set[0]].mag = 1;
	statusShow(-1,-1,1);
	progressClearProgress();
	progressShowStatus(PROGRESS_CALCULATING);
	GWerase(STATUS_CENTER_COORD,1);
	statusCenterCoord(&mandel,set[0]);
	goto REDO;
	break;
      case 3:
	//GWdelbmp(mandel.bmp2[set[0]]);
	progressClearProgress();
	progressShowStatus(PROGRESS_CALCULATING);
	GWerase(STATUS_CENTER_COORD,1);
	statusCenterCoord(&mandel,set[0]);
	goto REDO;
	break;

      default:
	break;
      }
      break;
    default:
      break;
    }

    GWmouse(NULL,&x,&y);
    if(x < 0 || x > DRAW_RECT_PIXEL || y < 0 || y > DRAW_RECT_PIXEL){
      if(flgRedrawRect){
	flgRedrawRect = 0;
	GWerase(MAG_RECT_OGN,1);
      }
      goto THRU_MAIN_A;
    }
    if(x + magRect > DRAW_RECT_PIXEL){
      x = DRAW_RECT_PIXEL - magRect - 1.f;
    }else if(x - magRect < 0){
      x = 0 + magRect;
    }
    if(y + magRect > DRAW_RECT_PIXEL){
      y = DRAW_RECT_PIXEL - magRect - 1.f;
    }else if(y - magRect < 0){
      y = 0 + magRect;
    }
    if(x == prevX && y == prevY){
      goto THRU_MAIN_A;
    }
    flgRedrawRect = 1;
    GWsetogn(0,-MAG_RECT_OGN);
    GWerase(MAG_RECT_OGN,1);
    GWsetpen(-1,-1,-1,GWX_NOT);
    GWsetbrs(-1,GWB_NULL,GWB_NULL);
    GWrect(x-magRect,y-magRect,x+magRect,y+magRect);
    GWflush(MAG_RECT_OGN);
  THRU_MAIN_A:
    prevX = x;
    prevY = y;
    Sleep(0x10);
  }

  //  while(1){
    /*
    countLine = 0;
    for(i = 0; i < param.threadNum;i++){
      countLine += (mandel.pReport + i)->haveDone;
    }
    progressBarProgress(countLine * 100 / PIXEL_X);

    if(countLine > prevCount){
      GWselvp(MAIN_VPORT);
      GWsetogn(0,-MAIN_BMP_OGN);
      GWsetbmp(a,DRAW_RECT_PIXEL,DRAW_RECT_PIXEL,2,1,GWO_BOTTOMLEFT);

      //GWpokebmp(a,0,PIXEL_X * PIXEL_X,GWY_SRCPAINT,mandel.brot[set[0]]);

      for(i = 0;i < param.threadNum;i++){
	if((mandel.pReport + i)->lastLineDraw != (mandel.pReport + i)->haveDone){
	  //static void poke(mandel *mandel,int threadId,int bmp);	  
	  brotPoke(&mandel,i,PIXEL_X / param.threadNum,set[0],a);
	  (mandel.pReport + i)->lastLineDraw = (mandel.pReport + i)->haveDone;
	}
      }

      GWputbmp(a,0,0,1);
      GWflush(-MAIN_BMP_OGN);
    }

    if(countLine == PIXEL_X){
      break;
    }
    prevCount = countLine;
    */
    //Sleep(10);
  //  }

#ifdef DEBUG
  /* DEBUG_dumpBrot(mandel.brot[0]); */
#endif
 END:
  
  freeMandel(&mandel);
  gradEnd();
  GWquitx(1);
  return 0;
};

/****************************************
 param\̂̏
****************************************/
static void initParam(pArgParam param){
  param->threadNum = threadGetLogicalProcessorNum() * 2;
  param->isWhiteBackColor = 0;
  param->iterateNum = DEFAULT_MAX_ITERATION;

  return;
}

/****************************************
 arg̎擾

 return:
 : 0
 s: -1
 ***************************************/
static int getParam(int argc,char **argv,pArgParam param){
  int ret;
  int i;
  const static char validParam[]={'t','v','n',};
  int validParamNum = sizeof(validParam) / sizeof(char);
  int num;

  if(argc == 1){
    ret = 0;
  }else if(argc > 0 && argc <= validParamNum + 1){
    for(i = 1;i < argc;i++){
      if(*argv[i] != '-'){
	ret = -1;
	goto checkArg_err;
      }
      if(!isValidOption(*(argv[i] + 1),validParam,validParamNum)){
	ret = -1;
	goto checkArg_err;
      }
    }

    for(i = 1;i < argc;i++){
      switch(*(argv[i] + 1)){
	  
      case 't':
	num = getArgNum(argv[i] + 2);
	if( num > THREADS_MAX || num < 1){
	  fprintf(stderr,"-tN: N must be between [1, %d]\n",THREADS_MAX);
	  ret = -1;
	  goto checkArg_err;
	}
	param->threadNum = num;
	break;
	
      case 'v':
	if(*(argv[i] + 2) != '\0'){
	  ret = -1;
	  goto checkArg_err;
	}
	param->isWhiteBackColor = 1;
	  break;
	  
      case 'n':
	num = getArgNum(argv[i] + 2);
	  if(num > SHRT_MAX || num < 1){
	    fprintf(stderr,"-nN: N must be between [1, %d]\n",SHRT_MAX);
	    ret = -1;
	    goto checkArg_err;
	  }
	  param->iterateNum = num;
	  break;
	  
#if 0
      default:
	ret = -1;
	goto checkArg_err;
	break;
#endif
      }
    }
    ret = 0;
  }else{
    fprintf(stderr,"Too many arguments\n");
    ret = -1;
  }
  
 checkArg_err:
  return ret;
}

/*********************************************
 𐔒lɂĕԂ
 
 return:
 : 擾
 s: 0
 ********************************************/
static int getArgNum(char *strNum){
  unsigned int num;
  int ret;
  unsigned int i;
  unsigned int strLen;
  
  num = atoi(strNum);

  if(num <= 0){
    ret = 0;
  }else{
    /* rŐlȊOꍇ͂ : 12a3 */
    strLen = (unsigned int)strlen(strNum);
    for(i = 0; i < strLen;i++){
      if(!isdigit(strNum[i])){
	ret = 0;
	goto getArgNum_err;
      }
    }
    ret = num;
  }

 getArgNum_err:
  return ret;
}

/**********************************************
 IvVL`FbN

 return:
 L: 1
 : 0
 *********************************************/
static int isValidOption(char opt,const char *validParam,int validParamNum){
  int i;
  int ret = 0;

  for(i = 0;i < validParamNum;i++){
    if(opt == *validParam){
      ret = 1;
      break;
    }
    validParam++;
  }

  return ret;
}

/*******************************************
 usage
*******************************************/
static void usage(void){
  printf("Usage: mandel.exe [option]\n");
  printf("[option]\n");
  printf("-nNUM\tset max iteration NUM\n");
  printf("-tNUM\tuse NUM-threads for iterate task\n");
  printf("-v\tswitch foreground and background\n");

  return;
}

/*********************************************
 ^XNL[ɒǉ
 return:
 : 0
 s: -1
 ********************************************/
static int throwTask(int iterateNum,int threadNum,int isWhiteBackColor,mandel *mandel){
  pTaskNode pNode;
  pCalcTask pTask;
  int i,j;

#ifdef USE_FLOAT128
  __float128 delta = (mandel->initialWidth / mandel->oct[set[0]].mag) / PIXEL_X;
  __float128 deltaHalf = delta / 2.0Q;
  __float128 x,y;
#else
  double delta = (mandel->initialWidth / mandel->oct[set[0]].mag) / PIXEL_X;
  double deltaHalf = delta / 2.0;
  double x,y;
#endif

  int lineForThread = PIXEL_X / threadNum;
  int lastLineAdd = PIXEL_X % threadNum;

  memset(mandel->pReport,0,sizeof(report) * threadNum);
  x = mandel->oct[set[0]].centerX - (PIXEL_X / 2) * delta + deltaHalf;
  y = mandel->oct[set[0]].centerY + (PIXEL_X / 2) * delta - deltaHalf;
  for(i = 0;i < threadNum;i++){
    if((pTask = malloc(sizeof(calcTask))) == NULL){
      return -1;
    }
    if((pNode = malloc(sizeof(taskNode))) == NULL){
      free(pTask);
      return -1;
    }
    pTask->x = x;
    pTask->y = y;
    for(j = 0;j < lineForThread;j++){
      y -= delta;
    }

    pTask->line = lineForThread + ((i == threadNum - 1)? lastLineAdd:0);
    pTask->delta = delta;
    pTask->brot = mandel->brot[set[0]];
    pTask->brot += i * PIXEL_X * lineForThread;
    pTask->iterate = iterateNum;
    (mandel->pReport + i)->startLine = i * lineForThread;
    pTask->pRep = mandel->pReport + i;
    
    pNode->pTask = pTask;
    mListAdd(&mandel->taskQ,(mNode *)pNode);
  }
  mandel->threadNum = i;
  mandel->isWhiteBackColor = isWhiteBackColor;
  
  return 0;
}

/*********************************************
 DEBUGp
 param\̂\
 ********************************************/
#ifdef DEBUG
static void DEBUG_showParam(pArgParam param){
  fprintf(stderr,"DEBUG_showParam called\n");
  fprintf(stderr,"param->threadNum = %d\n",param->threadNum);
  fprintf(stderr,"param->isWhiteBackColor = %d\n",param->isWhiteBackColor);
  fprintf(stderr,"param->iterateNum = %d\n",param->iterateNum);

  return;
}
#endif

/*********************************************
 DEBUGp
 񐔂\
 ********************************************/
#ifdef DEBUG
void DEBUG_dumpBrot(int *brot){
  int i,j;

  for(i = 0;i < 512;i++){
    for(j = 0;j < 128;j++){
      printf("%d ",*brot++);
    }
  }
  printf("ended!\n");

  return;
}
#endif

/****************************************
 efƂ̔L^Ă̈m
 return:
 : 0
 s: -1
 ***************************************/
static int allocBrot(int **brot,int *set){
  int i;
  int pixNum = PIXEL_X * PIXEL_X;

  for(i = 0;i < 5;i++){
    brot[i] = NULL;
    set[i] = i;
  }
  for(i = 0;i < 5;i++){
    brot[i] = malloc(sizeof(int)*pixNum);
    if(brot[i] == NULL){
      return -1;
    }
    zeroFillBrot(brot,i);
  }

  return 0;
}

/****************************************
 L^̈
 return:
 Ȃ
 ***************************************/
static void freeBrot(int **brot){
  int i;
  
  for(i = 0;i < 5;i++){
    if(brot[i] != NULL){
      free(brot[i]);
      brot[i] = NULL;
    }
  }

  return;
}

/****************************************
 L^̈0Ŗ߂
 return:
 Ȃ
 ***************************************/
static void zeroFillBrot(int **brot,int i){
  int pixNum = PIXEL_X * PIXEL_X;
  memset(brot[i],0,sizeof(int)*pixNum);
  return;
}

/****************************************
 `ʒuݒ
 return:
 Ȃ
 ***************************************/
static int initializeMandel(mandel *mandel,int threadNum){
  memset(mandel->oct,0,sizeof(oct) * 5);
  mandel->oct[set[0]].centerX = INITIAL_CENTER_X;
  mandel->oct[set[0]].centerY = INITIAL_CENTER_Y;
  mandel->initialWidth = INITIAL_WIDTH;
  mandel->oct[set[0]].mag = 1;
  mandel->pReport = NULL;
  grDrawMutex = NULL;
  nodeMutex = NULL;
  eventFlag = NULL;
  mandel->bmp = NULL;
  mandel->bmp2 = NULL;
  mandel->pList = NULL;
  mandel->pNode = NULL;
  mandel->pixBuff = NULL;

  if(allocBrot(mandel->brot,set) != 0){
    fprintf(stderr,"failed allocating memory-1\n");
    goto error_initializeMandel;
  }
  mListInit(&mandel->taskQ);
  if((mandel->pReport = malloc(sizeof(report) * threadNum)) == NULL){
    fprintf(stderr,"failed allocating memory-2\n");
    goto error_initializeMandel;
  }
  if((grDrawMutex = threadCreateMutex()) == NULL){
    fprintf(stderr,"failed allocating memory-3\n");
    goto error_initializeMandel;
  }
  if((nodeMutex = threadCreateMutex()) == NULL){
    fprintf(stderr,"failed allocating memory-4\n");
    goto error_initializeMandel;
  }
  if((eventFlag = threadCreateSemaphore(1)) == NULL){
    fprintf(stderr,"failed allocating memory-5\n");
    goto error_initializeMandel;
  }
  if((mandel->bmp = calloc(PIXEL_X,sizeof(int))) == NULL){
    fprintf(stderr,"failed allocating memory-6\n");
    goto error_initializeMandel;
  }
  if((mandel->bmp2 = calloc(5,sizeof(int))) == NULL){
    fprintf(stderr,"failed allocating memory-7\n");
    goto error_initializeMandel;
  }
  if((mandel->pList = malloc(sizeof(mDesc))) == NULL){
    fprintf(stderr,"failed allocating memory-8\n");
    goto error_initializeMandel;
  }
  mListInit(mandel->pList);
  if((mandel->pNode = calloc(PIXEL_X,sizeof(lineDrawNode))) == NULL){
    fprintf(stderr,"failed allocating memory-9\n");
    goto error_initializeMandel;
  }
  if((mandel->pixBuff = calloc(PIXEL_X * PIXEL_X,sizeof(int))) == NULL){
    fprintf(stderr,"failed allocating memory-10\n");
    goto error_initializeMandel;
  }

  return 0;

 error_initializeMandel:
  freeBrot(mandel->brot); /* this func free ONLY allocated brot */
  if(mandel->pReport != NULL){
    free(mandel->pReport);
  }
  if(grDrawMutex != NULL){
    threadCloseHandle(grDrawMutex);
  }
  if(nodeMutex != NULL){
    threadCloseHandle(nodeMutex);
  }
  if(eventFlag != NULL){
    threadCloseHandle(eventFlag);
  }
  if(mandel->bmp != NULL){
    free(mandel->bmp);
  }
  if(mandel->bmp2 != NULL){
    free(mandel->bmp2);
  }
  if(mandel->pList != NULL){
    free(mandel->pList);
  }
  if(mandel->pNode != NULL){
    free(mandel->pNode);
  }
  if(mandel->pixBuff != NULL){
    free(mandel->pixBuff);
  }
  
  return -1;
}

static void freeMandel(mandel *mandel){
  if(mandel->brot !=NULL) freeBrot(mandel->brot);
  if(mandel->pReport != NULL) free(mandel->pReport);
  if(grDrawMutex != NULL) threadCloseHandle(grDrawMutex);
  if(nodeMutex != NULL) threadCloseHandle(nodeMutex);
  if(eventFlag != NULL) threadCloseHandle(eventFlag);
  if(mandel->bmp != NULL) free(mandel->bmp);
  if(mandel->bmp2 != NULL) free(mandel->bmp2);
  if(mandel->pList != NULL) free(mandel->pList);
  if(mandel->pNode != NULL) free(mandel->pNode);
  if(mandel->pixBuff != NULL) free(mandel->pixBuff);
  return;
}

static void watchProgressThread(mandel *mandel){
  int countLine = 0;
  int i,j;
  int countDrawLine;
  int lineTsk = PIXEL_X / mandel->threadNum;
  int offset;
  lineDrawNode *pNode;
  int haveDone,lastDraw;
  
  while(1){
    for(i = 0; i < mandel->threadNum ;i++){
      //countLine += (mandel->pReport + i)->haveDone;
      
      haveDone = (mandel->pReport + i)->haveDone;
      lastDraw = (mandel->pReport + i)->lastLineDraw;
      countDrawLine = haveDone - lastDraw;
      for(j = 0;j < countDrawLine;j++){
	offset = i * lineTsk * PIXEL_X + PIXEL_X * lastDraw + j * PIXEL_X;

	pNode = mandel->pNode + i * lineTsk + lastDraw + j;
	pNode->start = mandel->pixBuff + offset;
	//printf("%d\n",*(pNode->start));
	pNode->line = pNode - mandel->pNode;

	WaitForSingleObject(grDrawMutex,INFINITE);
	brotMakeBitmapBufferFor1Line(mandel->pixBuff + offset,mandel->brot[set[0]] + offset);
	*(mandel->bmp + pNode->line) = GWmakebmp(0,PIXEL_X,1,32,pNode->start);
	ReleaseMutex(grDrawMutex);

	WaitForSingleObject(nodeMutex,INFINITE);	/* critical section */
	mListAdd(mandel->pList,(pMNode)pNode);
	ReleaseMutex(nodeMutex);			/* end of critical section */
	(mandel->pReport + i)->lastLineDraw++;
	countLine++;
      }
    }
    //printf("coutLine = %d\n",countLine);
    if(countLine == PIXEL_X){
      break;
    }
    Sleep(0x20);
  }
#ifdef DEBUG
  printf("end Thread progress\n");
#endif
  return;
}

static void drawLineThread(mandel *mandel){
  int countDraw = 0;
  pLineDrawNode pNode;
  int *bmp;
  int gwyMode = mandel->isWhiteBackColor? GWY_INVERTNOT:GWY_SRCINVERT;

  while(1){
    WaitForSingleObject(grDrawMutex,INFINITE);
    WaitForSingleObject(nodeMutex,INFINITE);
    GWselvp(MAIN_VPORT);
    while((pNode = (pLineDrawNode)mListGet(mandel->pList)) != NULL){
      GWsetogn(0,-MAIN_BMP_OGN);
      bmp = mandel->bmp + pNode->line;
      GWsetbmp(*bmp,DRAW_RECT_PIXEL,1,gwyMode,0,GWO_BOTTOMLEFT);
      GWputbmp(*bmp,0,(PIXEL_X -1) - pNode->line,1);
      countDraw++;
    }
    progressBarProgress(countDraw * 100 / PIXEL_X);
    GWflush(0);
    ReleaseMutex(nodeMutex);
    ReleaseMutex(grDrawMutex);

    if(countDraw == PIXEL_X){
      goto endDrawLineThread;
    }	
    Sleep(0x40);
  }

 endDrawLineThread:
#ifdef DEBUG
  printf("end Thread drawline\n");
#endif
  ReleaseSemaphore(eventFlag,1,NULL);
  return;
}

static void delBitmapLine(mandel *mandel){
  int i;
  for(i = 0;i < PIXEL_X;i++){
    GWdelbmp(*(mandel->bmp + i));
  }
  return;
}

static void flipSet(int stackId){
  int tmp;
  tmp = set[0];
  set[0] = set[stackId];
  set[stackId] = tmp;
  return;
}

static void setCenter(mandel *mandel,float x,float y){
#ifdef USE_FLOAT128
  __float128 delta;
#else
  double delta;
#endif
  delta = (mandel->initialWidth / mandel->oct[set[0]].mag) /  PIXEL_X;
  
  mandel->oct[set[0]].centerX += delta*(x-((DRAW_RECT_PIXEL - 1)/2));
  mandel->oct[set[0]].centerY += delta*(y-((DRAW_RECT_PIXEL - 1)/2));;

  //printf("******* %Lf %Lf\n",mandel->centerX,mandel->centerY);
    
  return;
}

static void copyCt(mandel *mandel,int stackId){
  memcpy(mandel->oct + stackId,mandel->oct + set[0],sizeof(oct));		     
  return;
}
#ifdef DEBUG
static void DEBUG_dump__(mandel *mandel){
  int i;
  for(i=0;i<5;i++){
    printf("---mandel->oct[%d]\n",i);
    printf("centerX = %lf\n",mandel->oct[i].centerX);
    printf("centerY = %lf\n",mandel->oct[i].centerY);
    printf("mag = %lld\n",mandel->oct[i].mag);
    printf("color = %d\n",mandel->oct[i].color);
    printf("set[%d] = %d\n",i,set[i]);
  }
  return;
}
#endif
