#include <grwin.h>
#include "progressBar.h"
#include "mandelcommon.h"

#define PROGRESS_DISP_UNIT 2

static int nowPercent;

/**************************************
 Progressバーのビューポート、ワールド座標の設定
 return:
 0
 *************************************/
int progressBarInit(void){
  /* Progress Bar */
  GWselvp(INITIAL_VPORT);
  GWport(DRAW_RECT_MARGIN32 * 3 + DRAW_RECT_PIXEL + DRAW_RECT_INNER_MARGIN8 * 2 + DRAW_RECT_SUB_PIXEL + DRAW_RECT_INNER_MARGIN2 * 2,WINDOW_HEIGHT - DRAW_RECT_MARGIN16 - (DRAW_RECT_SUB_PIXEL + DRAW_RECT_INNER_MARGIN2 * 2) * 2,
	 DRAW_RECT_MARGIN32 * 3 + DRAW_RECT_PIXEL + DRAW_RECT_INNER_MARGIN8 * 2 + DRAW_RECT_SUB_PIXEL + DRAW_RECT_INNER_MARGIN2 * 2 + DRAW_RECT_BAR_WIDTH,WINDOW_HEIGHT - DRAW_RECT_MARGIN16 - (DRAW_RECT_SUB_PIXEL + DRAW_RECT_INNER_MARGIN2 * 2) * 2 + DRAW_RECT_PROGRESS_BAR_HEIGHT);
  GWindow(0,0,DRAW_PROGRESS_BAR_WIDTH,DRAW_RECT_PROGRESS_BAR_HEIGHT);
  GWsavevp(PROGRESS_BAR_VPORT);

  return 0;
}

/*************************************
 Progressバー表示部の枠を描く
 ************************************/
void progressBarRect(void){
  GWselvp(PROGRESS_BAR_VPORT);
  GWsetogn(0,-PROGRESSBAR_RECT_OGN);
  GWrect(0,0,DRAW_PROGRESS_BAR_WIDTH,DRAW_RECT_PROGRESS_BAR_HEIGHT);
  GWflush(PROGRESSBAR_RECT_OGN);

  return;
}

/*************************************
 プログレスバーの描画を引数の割合に設定
 return:
 何もしなかった: 1
 描画を行った: 0
 Error: -1
 ************************************/
int progressBarProgress(int percent){
  int i;
  int diffPer;
  
  diffPer = percent - nowPercent;

  if(percent < 0 || percent > 100){
    return -1;
  }
  if(diffPer < PROGRESS_DISP_UNIT){
    return 1;
  }
  
  GWselvp(PROGRESS_BAR_VPORT);
  GWsetogn(0,PROGRESS_OGN);
#ifndef DEBUG
  GWsetpen(GWC_FGCOLOR,GWL_SOLID,0,GWX_COPYPEN);
#else
  GWsetpen(GWC_LIGHTGRAY,GWL_SOLID,0,GWX_COPYPEN);
#endif
  GWsetbrs(GWC_LIGHTGRAY,GWB_SOLID,GWB_NULL);
  
  while(diffPer >= PROGRESS_DISP_UNIT){
    GWrect(nowPercent,0,(float)nowPercent + PROGRESS_DISP_UNIT,DRAW_RECT_PROGRESS_BAR_HEIGHT);
    nowPercent += PROGRESS_DISP_UNIT;
    diffPer -= PROGRESS_DISP_UNIT;
  }
  if(percent >= 100){
    progressShowStatus(PROGRESS_COMPLETED);
    /*
    progressClearProgress();
    GWsetogn(0,PROGRESS_OGN);
    GWsetbrs(GWC_DARK_GRAY,GWB_SOLID,GWB_NULL);
    GWrect(0,0,100.f,DRAW_RECT_PROGRESS_BAR_HEIGHT);
    */
  }
  
  return 0;
}

/***********************************
 プログレスバーをクリア
 **********************************/
void progressClearProgress(void){
  nowPercent = 0;
  GWerase(PROGRESS_OGN,1);
  return;
}

#define PROGRESSBAR_FONT_HEIGHT 16.f
#define PROGRESSBAR_FONT_NAME "Consolas"
#define PROGRESSBAR_FONT_SHIFT_UP 4.f
/***********************************
 ステータスを表示
 **********************************/
void progressShowStatus(int status){
  static char statusStr[][64] = {"Calculating...","Completed!",};
  int color;

  GWselvp(PROGRESS_BAR_VPORT);
  GWsetogn(0,-PROGRESSBAR_STR_OGN);
  GWerase(PROGRESSBAR_STR_OGN,1);
  switch(status){
  case PROGRESS_CALCULATING:	color = GWC_FGCOLOR; break;
  case PROGRESS_COMPLETED:	color = GWC_GREEN; break;
  default:			color = -1; break;
  }
  GWsettxt(PROGRESSBAR_FONT_HEIGHT,0,GWO_BOTTOMLEFT,color,-1,PROGRESSBAR_FONT_NAME,strlen(PROGRESSBAR_FONT_NAME));
  GWputtxt(0,DRAW_RECT_PROGRESS_BAR_HEIGHT + PROGRESSBAR_FONT_SHIFT_UP,statusStr[status],strlen(statusStr[status]));
  GWflush(PROGRESSBAR_STR_OGN);

  return;
}
