#include <grwin.h>
#include <stdio.h>
#include "mandelcommon.h"
#ifdef USE_FLOAT128
#include <quadmath.h>
#endif

static int mag = 1;
static int isMandel = 1;
#define FONT_NAME_STATUS "Consolas"

/*************************************************
 ステータスを表示
 変更がない場合、その引数を-1にすれば再描画しない
 ************************************************/
void statusShow(int threadNum,int maxIterate,unsigned long long mag){
  char tmpStr[64];
  static int first = 1;

  GWselvp(STATUS_AREA_VPORT);
  GWsettxt(FONT_HEIGHT_STATUS,0,GWO_BOTTOMLEFT,-1,-1,FONT_NAME_STATUS,strlen(FONT_NAME_STATUS));

  if(threadNum != -1){
    GWerase(STATUS_THREAD_NUM_OGN,1);
    GWsetogn(0,-STATUS_THREAD_NUM_OGN);
    GWsettxt(FONT_HEIGHT_STATUS,0,GWO_BOTTOMLEFT,-1,-1,FONT_NAME_STATUS,strlen(FONT_NAME_STATUS));
    sprintf(tmpStr,"Thread: %d",threadNum);
    GWputtxt(0,STATUS_AREA_HEIGHT - FONT_HEIGHT_STATUS,tmpStr,strlen(tmpStr));
    GWflush(STATUS_THREAD_NUM_OGN);
  }
  if(maxIterate != -1){
    GWerase(STATUS_MAX_ITERATE_OGN,1);
    GWsetogn(0,-STATUS_MAX_ITERATE_OGN);
    GWsettxt(FONT_HEIGHT_STATUS,0,GWO_BOTTOMLEFT,-1,-1,FONT_NAME_STATUS,strlen(FONT_NAME_STATUS));
    sprintf(tmpStr,"Iterate: %d",maxIterate);
    GWputtxt(0,STATUS_AREA_HEIGHT - FONT_HEIGHT_STATUS * 2,tmpStr,strlen(tmpStr));
    GWflush(STATUS_MAX_ITERATE_OGN);
  }

  if(mag != -1){
    GWerase(STATUS_MAG_OGN,1);
    GWsetogn(0,-STATUS_MAG_OGN);
    GWsettxt(FONT_HEIGHT_STATUS,0,GWO_BOTTOMLEFT,-1,-1,FONT_NAME_STATUS,strlen(FONT_NAME_STATUS));
    sprintf(tmpStr,"Mag: %llux",mag);

    GWputtxt(0,STATUS_AREA_HEIGHT - FONT_HEIGHT_STATUS * 3,tmpStr,strlen(tmpStr));
    //GWsettxt(FONT_HEIGHT_STATUS,0,GWO_BOTTOMRIGHT,-1,-1,FONT_NAME_STATUS,strlen(FONT_NAME_STATUS));
    //sprintf(tmpStr,"%llux",mag);
    //GWputtxt(STATUS_AREA_WIDTH,STATUS_AREA_HEIGHT - FONT_HEIGHT_STATUS * 4,tmpStr,strlen(tmpStr));
    GWflush(STATUS_MAG_OGN);
  }
  
#ifdef USE_FLOAT128
  if(first){
    GWsetogn(0,-STATUS_FLOAT128_OGN);
    GWsettxt(FONT_HEIGHT_STATUS,0,GWO_BOTTOMLEFT,-1,-1,FONT_NAME_STATUS,strlen(FONT_NAME_STATUS));
    sprintf(tmpStr,"Using FLOAT128");
    GWputtxt(0,STATUS_AREA_HEIGHT - FONT_HEIGHT_STATUS * 4,tmpStr,strlen(tmpStr));
    GWflush(STATUS_FLOAT128_OGN);
  }
#endif
  if(first){
    GWsetogn(0,-MANDELBROT_STR_OGN);
    GWsettxt(FONT_HEIGHT_STATUS,0,GWO_BOTTOMLEFT,-1,-1,FONT_NAME_STATUS,strlen(FONT_NAME_STATUS));
    sprintf(tmpStr,"  Mandelbrot");
    GWputtxt(0,STATUS_AREA_HEIGHT - FONT_HEIGHT_STATUS * 5,tmpStr,strlen(tmpStr));
    GWflush(MANDELBROT_STR_OGN);
  }
  if(first){
    GWsetogn(0,-JULIA_STR_OGN);
    GWsettxt(FONT_HEIGHT_STATUS,0,GWO_BOTTOMLEFT,-1,-1,FONT_NAME_STATUS,strlen(FONT_NAME_STATUS));
    sprintf(tmpStr,"  Julia");
    GWputtxt(0,STATUS_AREA_HEIGHT - FONT_HEIGHT_STATUS * 6,tmpStr,strlen(tmpStr));
    GWflush(JULIA_STR_OGN);
  }
  first = 0;

  return;
}

/***********************************************
 中心の座標を表示
 **********************************************/
void statusCenterCoord(mandel* mandel,int set){
#define FONT_HEIGHT_CENTER_COORD 18.f
#define FONT_NAME_COORD "Consolas"
#define X_SHIFT_UP 4.f
#define Y_SHIFT_RIGHT 15.f
  char coord[64];

  GWselvp(MAIN_VPORT);
  GWsetogn(0,-STATUS_CENTER_COORD);

#ifdef USE_FLOAT128
  quadmath_snprintf(coord,sizeof(coord),"%1.34Qf",mandel->oct[set].centerX);
#else
  sprintf(coord,"%1.17lf",mandel->oct[set].centerX);
#endif
  GWsettxt(FONT_HEIGHT_CENTER_COORD,0,GWO_BOTTOM,-1,-1,FONT_NAME_COORD,strlen(FONT_NAME_COORD));
  GWputtxt(DRAW_RECT_PIXEL / 2 + DRAW_RECT_INNER_MARGIN8,-(DRAW_RECT_INNER_MARGIN8 + FONT_HEIGHT_CENTER_COORD) + X_SHIFT_UP,coord,strlen(coord));

#ifdef USE_FLOAT128
  quadmath_snprintf(coord,sizeof(coord),"%1.34Qf",mandel->oct[set].centerY);
#else
  sprintf(coord,"%1.17lf",mandel->oct[set].centerY);
#endif
  GWsettxt(FONT_HEIGHT_CENTER_COORD,90/360.f,GWO_BOTTOM,-1,-1,FONT_NAME_COORD,strlen(FONT_NAME_COORD));
  GWputtxt(-(DRAW_RECT_INNER_MARGIN8 + FONT_HEIGHT_CENTER_COORD) + Y_SHIFT_RIGHT,DRAW_RECT_PIXEL / 2 + DRAW_RECT_INNER_MARGIN8,coord,strlen(coord));

  GWflush(STATUS_CENTER_COORD);
  return;
  
}

void statusSetCheck(int isMandel){
  GWselvp(STATUS_AREA_VPORT);
  GWsetogn(0,-CHECK_SET_OGN);
  GWerase(CHECK_SET_OGN,1);
  GWsettxt(FONT_HEIGHT_STATUS,0,GWO_BOTTOMLEFT,-1,-1,FONT_NAME_STATUS,strlen(FONT_NAME_STATUS));
  GWputtxt(0,STATUS_AREA_HEIGHT - FONT_HEIGHT_STATUS * (isMandel? 5:6),">",1);
  GWflush(CHECK_SET_OGN);
  return;
}
